@@ -967,6 +967,8 @@ class OclStandardCsvToJsonConverter(OclCsvToJsonConverter):
                      'default': oclconstants.OclConstants.MAPPING_TARGET_INTERNAL},
                     {'resource_field': 'map_type', 'column_prefix': 'map_type',
                      'default': 'Same As'},
+                    {'resource_field': 'external_id', 'column_prefix': 'map_external_id',
+                     'required': False},
                     {'resource_field': 'from_concept_url', 'column_prefix': 'map_from_concept_url',
                      'required': False},
                     {'resource_field': 'from_concept_id', 'column_prefix': 'map_from_concept_id',
@@ -1018,6 +1020,8 @@ class OclStandardCsvToJsonConverter(OclCsvToJsonConverter):
                      'default': oclconstants.OclConstants.MAPPING_TARGET_EXTERNAL},
                     {'resource_field': 'map_type', 'column_prefix': 'extmap_type',
                      'default': 'Same As'},
+                    {'resource_field': 'external_id', 'column_prefix': 'extmap_external_id',
+                     'required': False},
                     {'resource_field': 'from_concept_url', 'required': False,
                      'column_prefix': 'extmap_from_concept_url'},
                     {'resource_field': 'from_concept_id', 'column_prefix': 'extmap_from_concept_id',
@@ -1101,6 +1105,8 @@ class OclStandardCsvToJsonConverter(OclCsvToJsonConverter):
                 {'resource_field': 'map_target', 'column': 'map_target',
                  'default': oclconstants.OclConstants.MAPPING_TARGET_INTERNAL},
                 {'resource_field': 'map_type', 'column': 'map_type', 'default': 'Same As'},
+                {'resource_field': 'external_id', 'column': 'external_id',
+                 'required': False},
                 {'resource_field': 'from_concept_url', 'required': False,
                  'column': ['map_from_concept_url', 'from_concept_url']},
                 {'resource_field': 'from_concept_id', 'required': False,
@@ -1143,6 +1149,8 @@ class OclStandardCsvToJsonConverter(OclCsvToJsonConverter):
                 {'resource_field': 'map_target', 'column': 'map_target',
                  'default': oclconstants.OclConstants.MAPPING_TARGET_EXTERNAL},
                 {'resource_field': 'map_type', 'column': 'map_type', 'default': 'Same As'},
+                {'resource_field': 'external_id', 'column': 'external_id',
+                 'required': False},
                 {'resource_field': 'from_concept_url', 'required': False,
                  'column': ['map_from_concept_url', 'from_concept_url']},
                 {'resource_field': 'from_concept_id', 'required': False,
