@@ -32,7 +32,8 @@ import oclresourcelist
 
 class OclImportError(Exception):
     """ Base exception for this module """
-    pass
+    def __str__(self):
+        return '%s %s' % (self.message, self.expression)
 
 
 class UnexpectedStatusCodeError(OclImportError):
@@ -185,7 +186,7 @@ class OclImportResults(object):
                         status_code_int = int(status_code)
                     except Exception:
                         status_code_int = 0
-                    if status_code == 'skip':
+                    if status_code == OclImportResults.SKIP_KEY:
                         return True
                     elif status_code_int and status_code_int >= 300:
                         return True
@@ -454,13 +455,13 @@ class OclFlexImporter(object):
     DEFAULT_REFERENCE_BATCH_SIZE = 0
 
     # Constants for import action types
-    ACTION_TYPE_DELETE = 'delete'
+    ACTION_TYPE_DELETE = 'DELETE'
     ACTION_TYPE_CREATE_OR_UPDATE = 'CREATE_OR_UPDATE'
-    ACTION_TYPE_CREATE = 'new'
-    ACTION_TYPE_UPDATE = 'update'
-    ACTION_TYPE_RETIRE = 'retire'
-    ACTION_TYPE_OTHER = 'other'
-    ACTION_TYPE_SKIP = 'skip'
+    ACTION_TYPE_CREATE = 'NEW'
+    ACTION_TYPE_UPDATE = 'UPDATE'
+    ACTION_TYPE_RETIRE = 'RETIRE'
+    ACTION_TYPE_OTHER = 'OTHER'
+    ACTION_TYPE_SKIP = 'SKIP'
     ACTION_TYPE_DEFAULT = ACTION_TYPE_CREATE_OR_UPDATE
 
     # All defined action types used for importing, logging, and future functionality
@@ -482,6 +483,17 @@ class OclFlexImporter(object):
         ACTION_TYPE_UPDATE,
     ]
 
+    # Constants for URL types
+    URL_TYPE_RESOURCE_TYPE_ONLY = 'resource_type_only'
+    URL_TYPE_RESOURCE_TYPE_AND_ID = 'resource_type_and_id'
+
+    # Constants for mapping supported import action types to url types
+    MAP_ACTION_TYPE_TO_URL_TYPE = {
+        ACTION_TYPE_DELETE: URL_TYPE_RESOURCE_TYPE_AND_ID,
+        ACTION_TYPE_CREATE: URL_TYPE_RESOURCE_TYPE_ONLY,
+        ACTION_TYPE_UPDATE: URL_TYPE_RESOURCE_TYPE_AND_ID
+    }
+
     # Constants for HTTP methods
     HTTP_METHOD_POST = "POST"
     HTTP_METHOD_PUT = "PUT"
@@ -511,7 +523,7 @@ class OclFlexImporter(object):
             ],
             OBJ_DEF_ATTR_HTTP_IMPORT_METHOD: {
                 ACTION_TYPE_CREATE: HTTP_METHOD_POST,
-                ACTION_TYPE_UPDATE: HTTP_METHOD_PUT,
+                ACTION_TYPE_UPDATE: HTTP_METHOD_POST,
                 ACTION_TYPE_DELETE: HTTP_METHOD_DELETE
             }
         },
@@ -588,7 +600,7 @@ class OclFlexImporter(object):
             "has_collection": True,
             "allowed_fields": ["data"],
             OBJ_DEF_ATTR_HTTP_IMPORT_METHOD: {
-                ACTION_TYPE_CREATE: HTTP_METHOD_POST,
+                ACTION_TYPE_CREATE: HTTP_METHOD_PUT,
                 ACTION_TYPE_UPDATE: None,
                 ACTION_TYPE_DELETE: HTTP_METHOD_DELETE
             }
@@ -655,7 +667,7 @@ class OclFlexImporter(object):
         self.skip_line_count = False
         self.reference_batch_size = reference_batch_size
         self.import_results = None
-        self.cache_obj_exists = {}
+        self._cache_obj_exists = {}
 
         # Prepare the headers
         self.api_headers = {
@@ -693,10 +705,11 @@ class OclFlexImporter(object):
 
     def process(self):
         """
-        Imports a JSON-lines file using OCL API
+        Import a list of OCL-formatted JSON resources using the OCL API
         :return: int Number of JSON lines processed
         """
 
+        # Start timer
         start_time = time.time()
 
         # Display global settings
@@ -704,17 +717,20 @@ class OclFlexImporter(object):
             self.log_settings()
 
         # Count lines
-        num_lines = 0
+        total_lines = 0
         if not self.skip_line_count:
-            num_lines = len(self.input_list)
+            total_lines = len(self.input_list)
 
-        # Loop through each JSON object in the file
-        self.import_results = OclImportResults(total_lines=num_lines)
+        # Loop through each JSON object
+        self.import_results = OclImportResults(total_lines=total_lines)
         obj_def_keys = self.obj_def.keys()
         count = 0
         num_processed = 0
         num_skipped = 0
         for json_line_obj in self.input_list:
+            # Copy the object so that the original remains untouched
+            json_line_obj = json_line_obj.copy()
+
             json_line_raw = json.dumps(json_line_obj)
             if self.limit > 0 and count >= self.limit:
                 break
@@ -753,22 +769,21 @@ class OclFlexImporter(object):
                 time.sleep(self.import_delay)
 
         self.import_results.elapsed_seconds = time.time() - start_time
-
         return count
 
     def does_object_exist(self, obj_url, use_cache=True):
         """ Returns whether a resource at the specified URL already exists. """
 
         # If we already know the resource exists, then just return True
-        if use_cache and obj_url in self.cache_obj_exists and self.cache_obj_exists[obj_url]:
+        if use_cache and obj_url in self._cache_obj_exists and self._cache_obj_exists[obj_url]:
             return True
 
         # Object existence not cached, so use OCL API to check if it exists
         request_existence = requests.head(self.api_url_root + obj_url, headers=self.api_headers)
         if request_existence.status_code == requests.codes.ok:
-            self.cache_obj_exists[obj_url] = True
+            self._cache_obj_exists[obj_url] = True
             return True
-        elif request_existence.status_code == requests.codes.not_found:
+        elif request_existence.status_code >= 400 and request_existence.status_code < 500:
             return False
         else:
             raise UnexpectedStatusCodeError(
@@ -889,11 +904,15 @@ class OclFlexImporter(object):
 
     def process_object(self, obj_type, obj):
         """ Processes an individual object in the import file """
+
         # Grab the object ID
         obj_id = ''
         if 'id_field' in self.obj_def[obj_type] and self.obj_def[obj_type]['id_field'] in obj:
             obj_id = obj[self.obj_def[obj_type]['id_field']]
 
+        # Determine initial action type
+        action_type = obj.get(OclFlexImporter.ATTR_ACTION, OclFlexImporter.ACTION_TYPE_DEFAULT)
+
         # Determine whether this resource has an owner, source, or collection
         has_owner = False
         has_source = False
@@ -932,13 +951,15 @@ class OclFlexImporter(object):
                         obj_owner)
                 else:
                     raise InvalidOwnerError(
-                        obj, "Valid owner information required for object of type '%s'" % obj_type)
+                        obj, "Valid owner information required for object of type '%s'. '%s' owner type provided." % (
+                            obj_owner_type, obj_type))
             elif has_source and 'source_url' in obj and obj['source_url']:
                 # Extract owner info from the source URL
-                obj_owner_url = obj['source_url'][:self.find_nth(obj['source_url'], '/', 3) + 1]
+                obj_owner_url = obj['source_url'][:OclFlexImporter.find_nth(
+                    obj['source_url'], '/', 3) + 1]
             elif has_collection and 'collection_url' in obj and obj['collection_url']:
                 # Extract owner info from the collection URL
-                obj_owner_url = obj['collection_url'][:self.find_nth(
+                obj_owner_url = obj['collection_url'][:OclFlexImporter.find_nth(
                     obj['collection_url'], '/', 3) + 1]
             else:
                 raise InvalidOwnerError(
@@ -1006,7 +1027,9 @@ class OclFlexImporter(object):
 
         # Display some debug info
         if self.verbosity >= 1:
-            self.log("**** Importing " + obj_type + ": " + self.api_url_root + obj_url + " ****")
+            self.log("**** %s %s: %s%s ****" % (
+                action_type, obj_type, self.api_url_root, obj_url))
+            # self.log("**** Importing " + obj_type + ": " + self.api_url_root + obj_url + " ****")
         if self.verbosity >= 2:
             self.log("** Allowed Fields: **", json.dumps(obj))
             self.log("** Removed Fields: **", json.dumps(obj_not_allowed))
@@ -1093,30 +1116,53 @@ class OclFlexImporter(object):
         # Perform the action: Create/update/delete the object
         try:
             self.import_resource(
-                obj_type=obj_type, obj_id=obj_id, obj_owner_url=obj_owner_url,
-                obj_repo_url=obj_repo_url, obj_url=obj_url, new_obj_url=new_obj_url,
-                obj_already_exists=obj_already_exists, obj=obj, obj_not_allowed=obj_not_allowed,
-                query_params=query_params)
+                action_type=action_type, obj_type=obj_type, obj_id=obj_id,
+                obj_owner_url=obj_owner_url, obj_repo_url=obj_repo_url, obj_url=obj_url,
+                new_obj_url=new_obj_url, obj_already_exists=obj_already_exists, obj=obj,
+                obj_not_allowed=obj_not_allowed, query_params=query_params)
         except requests.exceptions.HTTPError as err:
-            self.log("ERROR: ", err)
+            self.log("ERROR: ", str(err))
+        except UnsupportedActionType as err:
+            self.log("ERROR: ", str(err))
 
-    def import_resource(self, obj_type='', obj_id='', obj_owner_url='', obj_repo_url='',
-                        obj_url='', new_obj_url='', obj_already_exists=False, obj=None,
-                        obj_not_allowed=None, query_params=None):
-        """ Imports a resource as a create, update, or delete request to the OCL API """
+    def import_resource(self, action_type='', obj_type='', obj_id='', obj_owner_url='',
+                        obj_repo_url='', obj_url='', new_obj_url='', obj_already_exists=False,
+                        obj=None, obj_not_allowed=None, query_params=None):
+        """
+        Imports a resource as a create, update, or delete request to the OCL API.
+        :param action_type: One of the OclFlexImporter.IMPORT_ACTION_TYPES constants
+        :param obj_type: One of the OclConstants.RESOURCE_TYPES constants
+        :param obj_id: Unique identifier for the object
+        :param obj_repo_url: Relative URL of a resource's repository, if applicable, eg:
+            /orgs/MyOrg/sources/MySource/
+        :param obj_url: Relative URL of a resource including its identifier, eg:
+            /orgs/MyOrg/sources/MySource/concepts/MyConceptId/
+        :param new_obj_url: Reelative URL used to create a new object, excluding its identifier.
+            For example, for a concept it would be: /orgs/MyOrg/sources/MySource/concepts/
+        :param obj_already_exists: Whether an object already exists
+        :param obj: JSON object to be imported
+        :param obj_not_allowed: Fields removed from the object before importing
+        :param query_params: Additional parameters to be included in the import API request
+        """
 
-        # Determine the action type and URL
-        if (OclFlexImporter.ATTR_ACTION in obj and
-                obj[OclFlexImporter.ATTR_ACTION] in OclFlexImporter.IMPORT_ACTION_TYPES):
-            action_type = obj[OclFlexImporter.ATTR_ACTION]
-        else:
+        # Determine the action type
+        if not action_type:
             action_type = OclFlexImporter.ACTION_TYPE_DEFAULT
         if action_type == OclFlexImporter.ACTION_TYPE_CREATE_OR_UPDATE:
             if obj_already_exists:
                 action_type = OclFlexImporter.ACTION_TYPE_UPDATE
             else:
                 action_type = OclFlexImporter.ACTION_TYPE_CREATE
-        url = obj_url if obj_already_exists else new_obj_url
+        elif action_type not in OclFlexImporter.IMPORT_ACTION_TYPES:
+            raise UnsupportedActionType(obj, "Unsupported import action type '%s'" % action_type)
+
+        # Determine the request URL
+        if action_type in OclFlexImporter.MAP_ACTION_TYPE_TO_URL_TYPE:
+            url_type = OclFlexImporter.MAP_ACTION_TYPE_TO_URL_TYPE[action_type]
+            if url_type == OclFlexImporter.URL_TYPE_RESOURCE_TYPE_ONLY:
+                url = new_obj_url
+            elif url_type == OclFlexImporter.URL_TYPE_RESOURCE_TYPE_AND_ID:
+                url = obj_url
 
         # Determine which HTTP method to use
         obj_def = self.obj_def[obj_type]
@@ -1124,7 +1170,7 @@ class OclFlexImporter(object):
                 action_type in obj_def[OclFlexImporter.OBJ_DEF_ATTR_HTTP_IMPORT_METHOD]):
             method = obj_def[OclFlexImporter.OBJ_DEF_ATTR_HTTP_IMPORT_METHOD][action_type]
             if not method:
-                err_msg = 'Action type "%s" not supported for resource type "%s"' % (
+                err_msg = 'Action type "%s" not supported for resource type of "%s"' % (
                     action_type, obj_type)
                 raise UnsupportedActionType(obj, err_msg)
         else:
@@ -1136,6 +1182,10 @@ class OclFlexImporter(object):
         if query_params:
             url += '?' + urllib.urlencode(query_params)
 
+        # If delete, then clear the cache of object existence
+        if action_type == OclFlexImporter.ACTION_TYPE_DELETE:
+            self.clear_cache(obj_url=obj_url, do_clear_children=True)
+
         # Import the object
         if not self.test_mode:
             self.log(method, " ", self.api_url_root + url + '  ', json.dumps(obj))
@@ -1158,10 +1208,27 @@ class OclFlexImporter(object):
                 message=request_result.text)
             request_result.raise_for_status()
         else:
-            self.log("[TEST MODE] ", method, self.api_url_root + url + '  ', json.dumps(obj))
+            self.log("[TEST MODE] Action: %s, Method: %s, URL: %s%s -- %s" % (
+                action_type, method, self.api_url_root, url, json.dumps(obj)))
 
-    def find_nth(self, haystack, needle, n):
-        """ Find nth occurrence of a substring within a string """
+    def clear_cache(self, obj_url, do_clear_children=False):
+        """
+        Clear the object existence cache of the specified object URL. Optionally
+        also clear any occurrences of children of the specified object.
+        """
+        if not obj_url:
+            return
+        if not do_clear_children:
+            if obj_url in self._cache_obj_exists:
+                del self._cache_obj_exists[obj_url]
+        else:
+            for key in self._cache_obj_exists.keys():
+                if key[:len(obj_url)] == obj_url:
+                    del self._cache_obj_exists[key]
+
+    @staticmethod
+    def find_nth(haystack, needle, n):
+        """ Return index of nth occurrence of a substring within a string """
         start = haystack.find(needle)
         while start >= 0 and n > 1:
             start = haystack.find(needle, start+len(needle))
